/* copyright 1997 ondrej@idata.sk */
/* distributed under GPL */

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h> 

#include "play.h"

char buf[64 * 1024];

void wav_info(info , header)
char *info;
wave_header *header;
{
	fprintf(stderr ,"*********  %s  *********\n" , info);
	fprintf(stderr ,"FILE SIZE         : %u B\n" , header->length);
	fprintf(stderr ,"DATAPART SIZE     : %u B\n" , header->data_length);
 	fprintf(stderr ,"TYPE              : %u bit %s\n" , header->bit_p_spl ,
		(header->modus == 1) ? "mono" : "stereo");
	fprintf(stderr ,"SAMPLE RATE       : %u Hz\n" , header->sample_fq);
	fprintf(stderr ,"PLAY TIME         : %u s\n" ,
		header->data_length / header->byte_p_sec);
	fprintf(stderr ,"NUMBER of SAMPLES : %u\n" ,
		header->data_length / header->byte_p_spl);
	fprintf(stderr ,"*******************************\n");
}

void usage(pnm)
char *pnm;
{
	fprintf(stderr ,"Usage: %s [-h] -if $file [-of $file] [-info] [-bt $time_s] [-et $time_s] [-bp $start_spl] [-ep $endspl]\n\n" , pnm);
	fprintf(stderr ,"\t-h     - print this help and die\n");
	fprintf(stderr ,"\t-if	- input filename (\"-\" means stdin)\n");
	fprintf(stderr ,"\t-of	- output filename (\"-\" means stdout)\n"); 
	fprintf(stderr ,"\t-info - print some informations\n");
	fprintf(stderr ,"\t-bt	- begin time of cuted part in seconds\n");
	fprintf(stderr ,"\t-et   - end time of cuted part in seconds\n");
	fprintf(stderr ,"\t-bp   - begin sample number of cuted part\n");
	fprintf(stderr ,"\t-ep   - end sample number of cuted part\n");
	exit(0);
}

int main(argc , argv)
int argc;
char **argv;
{
	wave_header header , oheader;
	FILE *infile , *outfile;
	char *infn = NULL;
	char *outfn = NULL;
	long apos;
	long bpos = 0;
	long epos = 0;
	long btime = 0;
	long etime = 0;
	int i;
	int info = 0;

	for(i = 1 ; i < argc ; i++)
	{
		if (!strcmp(argv[i] , "-h"))
		{
			usage(argv[0]);
		} 
		else if (!strcasecmp(argv[i] , "-if"))
		{
			infn = argv[++i];
		}
		else if (!strcasecmp(argv[i] , "-of"))
		{
			outfn = argv[++i];
		}
		else if (!strcasecmp(argv[i] , "-bs"))
		{
			bpos = atoi(argv[++i]);
		}
		else if (!strcasecmp(argv[i] , "-es"))
		{
			epos = atoi(argv[++i]);
		}
		else if (!strcasecmp(argv[i] , "-bt"))
		{
			btime = atoi(argv[++i]);
		}
		else if (!strcasecmp(argv[i] , "-et"))
		{
			etime = atoi(argv[++i]);
		}
		else if (!strcasecmp(argv[i] , "-info"))
		{
			info = 1;
		}
		else { fprintf(stderr ,"unknovn parameter: %s\n" , argv[i]) ; exit(1);}
	}

	if (!infn) { fprintf(stderr ,"specify input file\n"); exit(1);}

	if (!(infile = (strcmp(infn , "-") ? fopen(infn , "r") : stdin)))
	{
		perror(infn);
		exit(1);
	}

	if (fread((void *) &header , 1 , sizeof(wave_header) , infile) != sizeof(wave_header))
	{
		fprintf(stderr ,"too short file\n");
		exit(1);
	}

	if (! header_check(&header)) exit(1);

	if (info) wav_info("input file" , &header);


	epos *= header.byte_p_spl;
	bpos *= header.byte_p_spl;

	if (!epos) epos = header.data_length;

	if (epos < 0) epos = header.data_length + epos;
	if (bpos < 0) bpos = header.data_length + bpos;

	if (btime)
		bpos = ((btime < 0) ? header.data_length : 0) + 
			btime * header.byte_p_sec;

	if (etime)
                epos = ((etime < 0) ? header.data_length : 0) +
                        etime * header.byte_p_sec;

	if (epos < bpos) {printf("end position befor start position\n"); exit(1);}

	if (bpos > header.data_length)
	{
		fprintf(stderr ,"starting past end\n");
		exit(1);
	}

        if (epos > header.data_length)
        {
                fprintf(stderr ,"ending past end - seting end position to EOF\n");
		epos = header.data_length;
        }

	if (info)
	{
		fprintf(stderr ,"\nCUTING FROM %u TO %u - TIME %u s\n\n" , 
			bpos / header.byte_p_spl , 
			epos / header.byte_p_spl ,
			(epos - bpos) / header.byte_p_sec);
	}

	memcpy((void *) &oheader , (void *) &header , sizeof(wave_header));

	oheader.data_length = epos - bpos;
	oheader.length = sizeof(wave_header) + oheader.data_length;

	if (info) wav_info("output file" , &oheader);

	if (!info && !outfn) { fprintf(stderr ,"specify output file\n"); exit(1);}

	if (!outfn) exit(0);

        if (!(outfile = (strcmp(outfn , "-") ? fopen(outfn , "w") : stdout)))
        {
                perror(outfn);
                exit(1);
        }

	fwrite(&oheader , 1 , sizeof(header) , outfile);

	fseek(infile , bpos , SEEK_CUR);	

	apos = bpos;

	while(!feof(infile) && epos > apos)
	{
		i = fread(buf , 1 , sizeof(buf) , infile);
		if (apos + i > epos) i = epos - apos;

		apos += i;

		if (fwrite(buf , 1 ,  i , outfile) != i)
		{
			ferror(outfile);
			exit(1);
		}
	}

	fclose(infile);
	fclose(outfile);
	
	exit(0); 
}

